@@ -33,6 +33,7 @@
 (require 'seq)
 (require 'vc)
 (require 'vc-git)
+(require 'log-view)
 (require 'ansi-color)
 (require 'iso8601)
 (require 'time-date)
@@ -508,12 +509,6 @@ If REV is not specified, revert the file as with `vc-jj-revert'."
 ;;   ;; TODO
 ;;   )
 
-(defvar log-view-message-re)
-(defvar log-view-file-re)
-(defvar log-view-font-lock-keywords)
-(defvar log-view-per-file-logs)
-(defvar log-view-expanded-log-entry-function)
-
 (defvar vc-jj--logline-re
   (rx
     line-start
@@ -657,17 +652,7 @@ If REV is not specified, revert the file as with `vc-jj-revert'."
 (defun vc-jj-root (file)
   "Return the root of the repository containing FILE.
 Return NIL if FILE is not in a jj repository."
-  ;; `default-directory' must be an absolute directory name ending
-  ;; with a slash, so we go to some lengths to guarantee this, even
-  ;; for e.g. (vc-jj-root ".")
-  (let* ((absolute-file (expand-file-name file))
-         (default-directory
-          (file-name-as-directory (if (file-directory-p absolute-file)
-                                      absolute-file
-                                    (file-name-directory absolute-file)))))
-    (with-temp-buffer
-      (when (= 0 (call-process vc-jj-program nil (list t nil) nil "root"))
-        (file-name-as-directory (buffer-substring (point-min) (1- (point-max))))))))
+  (vc-find-root file ".jj"))
 
 (defalias 'vc-jj-responsible-p #'vc-jj-root)
 
@@ -735,6 +720,8 @@ For jj, modify `.gitignore' and call `jj untrack' or `jj track'."
 The regex matches each line's commit information and captures
 four groups: change id, author, datetime, line number.")
 
+(declare-function vc-annotate-convert-time "vc-annotate" (&optional time))
+
 (defun vc-jj-annotate-time ()
   "Return the time for the annotated line."
   (and-let*
@@ -760,7 +747,7 @@ element, or can be a change id, in which case we query for the first
 line of its description."
   (let ((description
          (if (listp elem)
-             (second elem)
+             (cl-second elem)
            (vc-jj--command-parseable "log" "-r" elem "--no-graph" "-T" "self.description().first_line()"))))
     (format " %s" (propertize description 'face 'completions-annotations))))
 
@@ -844,6 +831,18 @@ the command to run, e.g., the semi-standard \"jj git push -c @-\"."
 
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("\\.jjdescription\\'" . log-edit-mode))
+;;;###autoload
+(add-hook 'vc-log-mode-hook #'vc-jj-ensure-log-edit-callback)
+;;;###autoload
+(add-hook 'log-edit-mode-hook #'vc-jj-ensure-log-edit-callback)
+;;;###autoload
+(defun vc-jj-ensure-log-edit-callback ()
+  "Set up `log-edit-callback' when editing jj commit messages."
+  (unless (bound-and-true-p log-edit-callback)
+    (setq-local log-edit-callback (lambda ()
+                                    (interactive)
+                                    (save-buffer)
+                                    (kill-buffer)))))
 
 (provide 'vc-jj)
 ;;; vc-jj.el ends here
